@@ -4,7 +4,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -19,14 +19,12 @@
 
 {
   INDEX i;
-  INDEX ix;
+  INDEX ix = 0;
 
   if (incX <= 0) {
     return;
   }
 
-  ix = OFFSET(N, incX);
-
   for (i = 0; i < N; i++) {
     REAL(X, ix) *= alpha;
     IMAG(X, ix) *= alpha;
