@@ -323,7 +323,7 @@ gsl_linalg_SV_decomp_mod (gsl_matrix * A,
 
   /* Convert A into an orthogonal matrix L */
 
-  for (j = N; j > 0 && j--;)
+  for (j = N; j-- > 0;)
     {
       /* Householder column transformation to accumulate L */
       double tj = gsl_vector_get (S, j);
@@ -533,8 +533,8 @@ gsl_linalg_SV_decomp_jacobi (gsl_matrix * A, gsl_matrix * Q, gsl_vector * S)
                   abserr_a = gsl_vector_get(S,j);
                   abserr_b = gsl_vector_get(S,k);
 
-                  sorted = (gsl_coerce_double(a) >= gsl_coerce_double(b));
-                  orthog = (fabs (p) <= tolerance * gsl_coerce_double(a * b));
+                  sorted = (GSL_COERCE_DBL(a) >= GSL_COERCE_DBL(b));
+                  orthog = (fabs (p) <= tolerance * GSL_COERCE_DBL(a * b));
                   noisya = (a < abserr_a);
                   noisyb = (b < abserr_b);
 
