@@ -1,10 +1,10 @@
 /* rng/default.c
  * 
- * Copyright (C) 1996, 1997, 1998, 1999, 2000 James Theiler, Brian Gough
+ * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 James Theiler, Brian Gough
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -63,10 +63,12 @@ gsl_rng_env_setup (void)
 
               if ((++i) % 4 == 0)
                 {
-                  putchar ('\n');
+                  fputc ('\n', stderr);
                 }
             }
 
+          fputc ('\n', stderr);
+
           GSL_ERROR_VAL ("unknown generator", GSL_EINVAL, 0);
         }
 
