@@ -1,10 +1,10 @@
 /* statistics/wvariance_source.c
  * 
- * Copyright (C) 1996, 1997, 1998, 1999, 2000 Jim Davies, Brian Gough
+ * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2007 Jim Davies, Brian Gough
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or (at
+ * the Free Software Foundation; either version 3 of the License, or (at
  * your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful, but
@@ -20,6 +20,10 @@
 static double 
 FUNCTION(compute,wvariance) (const BASE w[], const size_t wstride, const BASE data[], const size_t stride, const size_t n, const double wmean);
 
+static double 
+FUNCTION(compute,wtss) (const BASE w[], const size_t wstride, const BASE data[], const size_t stride, const size_t n, const double wmean);
+
+
 static double
 FUNCTION(compute,factor) (const BASE w[], const size_t wstride, const size_t n);
 
@@ -48,6 +52,29 @@ FUNCTION(compute,wvariance) (const BASE w[], const size_t wstride, const BASE da
   return wvariance ;
 }
 
+static double
+FUNCTION(compute,wtss) (const BASE w[], const size_t wstride, const BASE data[], const size_t stride, const size_t n, const double wmean)
+{
+  /* takes a dataset and finds the weighted sum of squares about wmean*/
+
+  long double wtss = 0 ;
+  size_t i;
+
+  /* find the sum of the squares */
+  for (i = 0; i < n; i++)
+    {
+      BASE wi = w[i * wstride];
+
+      if (wi > 0) {
+        const long double delta = (data[i * stride] - wmean);
+        wtss += wi * delta * delta;
+      }
+    }
+
+  return wtss ;
+}
+
+
 static double
 FUNCTION(compute,factor) (const BASE w[], const size_t wstride, const size_t n)
 {
@@ -125,3 +152,19 @@ FUNCTION(gsl_stats,wvariance) (const BASE w[], const size_t wstride, const BASE
   const double wmean = FUNCTION(gsl_stats,wmean) (w, wstride, data, stride, n);
   return FUNCTION(gsl_stats,wvariance_m)(w, wstride, data, stride, n, wmean);
 }
+
+double 
+FUNCTION(gsl_stats,wtss_m) (const BASE w[], const size_t wstride, const BASE data[], const size_t stride, const size_t n, const double wmean)
+{
+  const double wtss = FUNCTION(compute,wtss) (w, wstride, data, stride, n, wmean);
+  return wtss;
+}
+
+double 
+FUNCTION(gsl_stats,wtss) (const BASE w[], const size_t wstride, const BASE data[], const size_t stride, const size_t n)
+{
+  const double wmean = FUNCTION(gsl_stats,wmean) (w, wstride, data, stride, n);
+  return FUNCTION(gsl_stats,wtss_m)(w, wstride, data, stride, n, wmean);
+}
+
+
